# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_lic_global_optspecs
	string join \n v/verbose q/quiet c/config= h/help V/version
end

function __fish_lic_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_lic_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_lic_using_subcommand
	set -l cmd (__fish_lic_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c lic -n "__fish_lic_needs_command" -s c -l config -r -F
complete -c lic -n "__fish_lic_needs_command" -s v -l verbose -d 'Increase logging verbosity'
complete -c lic -n "__fish_lic_needs_command" -s q -l quiet -d 'Decrease logging verbosity'
complete -c lic -n "__fish_lic_needs_command" -s h -l help -d 'Print help'
complete -c lic -n "__fish_lic_needs_command" -s V -l version -d 'Print version'
complete -c lic -n "__fish_lic_needs_command" -f -a "gen" -d 'Generate a license file'
complete -c lic -n "__fish_lic_needs_command" -f -a "unapply" -d 'Remove license headers in source files'
complete -c lic -n "__fish_lic_needs_command" -f -a "apply" -d 'Apply license headers to source files'
complete -c lic -n "__fish_lic_needs_command" -f -a "init" -d 'Initialize a default configuration file'
complete -c lic -n "__fish_lic_needs_command" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c lic -n "__fish_lic_using_subcommand gen" -s a -l authors -d 'Author names and emails (In the format NAME:EMAIL; entries seperated by a comma. Email optional)' -r
complete -c lic -n "__fish_lic_using_subcommand gen" -s d -l date -d 'Date for the license copyright notice (defaults to the current year)' -r
complete -c lic -n "__fish_lic_using_subcommand gen" -l multiple -d 'Enable support for multiple licenses in the same project (Default is replace)'
complete -c lic -n "__fish_lic_using_subcommand gen" -s v -l verbose -d 'Increase logging verbosity'
complete -c lic -n "__fish_lic_using_subcommand gen" -s q -l quiet -d 'Decrease logging verbosity'
complete -c lic -n "__fish_lic_using_subcommand gen" -s h -l help -d 'Print help'
complete -c lic -n "__fish_lic_using_subcommand unapply" -s e -l exclude -d 'Regex pattern for files/directories to exclude. Applied during directory traversal' -r
complete -c lic -n "__fish_lic_using_subcommand unapply" -s D -l dry-run -d 'Run without modification. See what would be changed' -r -f -a "true\t''
false\t''"
complete -c lic -n "__fish_lic_using_subcommand unapply" -s A -l all -d 'Do not respect the git_ignore file (If present in directory) and other pattern defaults'
complete -c lic -n "__fish_lic_using_subcommand unapply" -s v -l verbose -d 'Increase logging verbosity'
complete -c lic -n "__fish_lic_using_subcommand unapply" -s q -l quiet -d 'Decrease logging verbosity'
complete -c lic -n "__fish_lic_using_subcommand unapply" -s h -l help -d 'Print help'
complete -c lic -n "__fish_lic_using_subcommand apply" -s a -l authors -d 'Author names and emails (In the format NAME:EMAIL; entries seperated by a comma. Email optional)' -r
complete -c lic -n "__fish_lic_using_subcommand apply" -s d -l date -d 'Date for the license copyright notice (defaults to the current year)' -r
complete -c lic -n "__fish_lic_using_subcommand apply" -s e -l exclude -d 'Regex pattern for files/directories to exclude. Applied during directory traversal' -r
complete -c lic -n "__fish_lic_using_subcommand apply" -l multiple -d 'Enable support for multiple licenses in the same project (Default is replace)'
complete -c lic -n "__fish_lic_using_subcommand apply" -s A -l all -d 'Do not respect the git_ignore file (If present in directory) and other pattern defaults'
complete -c lic -n "__fish_lic_using_subcommand apply" -s D -l dry-run -d 'Run without modification. See what would be changed'
complete -c lic -n "__fish_lic_using_subcommand apply" -l prefer-block -d 'When applying headers, which kind of comment token the user *wants* Completely possible line or block doesn\'t exist, in which case it falls back to the other'
complete -c lic -n "__fish_lic_using_subcommand apply" -s v -l verbose -d 'Increase logging verbosity'
complete -c lic -n "__fish_lic_using_subcommand apply" -s q -l quiet -d 'Decrease logging verbosity'
complete -c lic -n "__fish_lic_using_subcommand apply" -s h -l help -d 'Print help'
complete -c lic -n "__fish_lic_using_subcommand init" -s v -l verbose -d 'Increase logging verbosity'
complete -c lic -n "__fish_lic_using_subcommand init" -s q -l quiet -d 'Decrease logging verbosity'
complete -c lic -n "__fish_lic_using_subcommand init" -s h -l help -d 'Print help'
complete -c lic -n "__fish_lic_using_subcommand help; and not __fish_seen_subcommand_from gen unapply apply init help" -f -a "gen" -d 'Generate a license file'
complete -c lic -n "__fish_lic_using_subcommand help; and not __fish_seen_subcommand_from gen unapply apply init help" -f -a "unapply" -d 'Remove license headers in source files'
complete -c lic -n "__fish_lic_using_subcommand help; and not __fish_seen_subcommand_from gen unapply apply init help" -f -a "apply" -d 'Apply license headers to source files'
complete -c lic -n "__fish_lic_using_subcommand help; and not __fish_seen_subcommand_from gen unapply apply init help" -f -a "init" -d 'Initialize a default configuration file'
complete -c lic -n "__fish_lic_using_subcommand help; and not __fish_seen_subcommand_from gen unapply apply init help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
