
using namespace System.Management.Automation
using namespace System.Management.Automation.Language

Register-ArgumentCompleter -Native -CommandName 'lic' -ScriptBlock {
    param($wordToComplete, $commandAst, $cursorPosition)

    $commandElements = $commandAst.CommandElements
    $command = @(
        'lic'
        for ($i = 1; $i -lt $commandElements.Count; $i++) {
            $element = $commandElements[$i]
            if ($element -isnot [StringConstantExpressionAst] -or
                $element.StringConstantType -ne [StringConstantType]::BareWord -or
                $element.Value.StartsWith('-') -or
                $element.Value -eq $wordToComplete) {
                break
        }
        $element.Value
    }) -join ';'

    $completions = @(switch ($command) {
        'lic' {
            [CompletionResult]::new('-c', '-c', [CompletionResultType]::ParameterName, 'c')
            [CompletionResult]::new('--config', '--config', [CompletionResultType]::ParameterName, 'config')
            [CompletionResult]::new('-v', '-v', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('--verbose', '--verbose', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('-q', '-q', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('--quiet', '--quiet', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('-h', '-h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', '--help', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('-V', '-V ', [CompletionResultType]::ParameterName, 'Print version')
            [CompletionResult]::new('--version', '--version', [CompletionResultType]::ParameterName, 'Print version')
            [CompletionResult]::new('gen', 'gen', [CompletionResultType]::ParameterValue, 'Generate a license file')
            [CompletionResult]::new('unapply', 'unapply', [CompletionResultType]::ParameterValue, 'Remove license headers in source files')
            [CompletionResult]::new('apply', 'apply', [CompletionResultType]::ParameterValue, 'Apply license headers to source files')
            [CompletionResult]::new('init', 'init', [CompletionResultType]::ParameterValue, 'Initialize a default configuration file')
            [CompletionResult]::new('help', 'help', [CompletionResultType]::ParameterValue, 'Print this message or the help of the given subcommand(s)')
            break
        }
        'lic;gen' {
            [CompletionResult]::new('-a', '-a', [CompletionResultType]::ParameterName, 'Author names and emails (In the format NAME:EMAIL; entries seperated by a comma. Email optional)')
            [CompletionResult]::new('--authors', '--authors', [CompletionResultType]::ParameterName, 'Author names and emails (In the format NAME:EMAIL; entries seperated by a comma. Email optional)')
            [CompletionResult]::new('-d', '-d', [CompletionResultType]::ParameterName, 'Date for the license copyright notice (defaults to the current year)')
            [CompletionResult]::new('--date', '--date', [CompletionResultType]::ParameterName, 'Date for the license copyright notice (defaults to the current year)')
            [CompletionResult]::new('--multiple', '--multiple', [CompletionResultType]::ParameterName, 'Enable support for multiple licenses in the same project (Default is replace)')
            [CompletionResult]::new('-v', '-v', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('--verbose', '--verbose', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('-q', '-q', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('--quiet', '--quiet', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('-h', '-h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', '--help', [CompletionResultType]::ParameterName, 'Print help')
            break
        }
        'lic;unapply' {
            [CompletionResult]::new('-e', '-e', [CompletionResultType]::ParameterName, 'Regex pattern for files/directories to exclude. Applied during directory traversal')
            [CompletionResult]::new('--exclude', '--exclude', [CompletionResultType]::ParameterName, 'Regex pattern for files/directories to exclude. Applied during directory traversal')
            [CompletionResult]::new('-D', '-D ', [CompletionResultType]::ParameterName, 'Run without modification. See what would be changed')
            [CompletionResult]::new('--dry-run', '--dry-run', [CompletionResultType]::ParameterName, 'Run without modification. See what would be changed')
            [CompletionResult]::new('-A', '-A ', [CompletionResultType]::ParameterName, 'Do not respect the git_ignore file (If present in directory) and other pattern defaults')
            [CompletionResult]::new('--all', '--all', [CompletionResultType]::ParameterName, 'Do not respect the git_ignore file (If present in directory) and other pattern defaults')
            [CompletionResult]::new('-v', '-v', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('--verbose', '--verbose', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('-q', '-q', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('--quiet', '--quiet', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('-h', '-h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', '--help', [CompletionResultType]::ParameterName, 'Print help')
            break
        }
        'lic;apply' {
            [CompletionResult]::new('-a', '-a', [CompletionResultType]::ParameterName, 'Author names and emails (In the format NAME:EMAIL; entries seperated by a comma. Email optional)')
            [CompletionResult]::new('--authors', '--authors', [CompletionResultType]::ParameterName, 'Author names and emails (In the format NAME:EMAIL; entries seperated by a comma. Email optional)')
            [CompletionResult]::new('-d', '-d', [CompletionResultType]::ParameterName, 'Date for the license copyright notice (defaults to the current year)')
            [CompletionResult]::new('--date', '--date', [CompletionResultType]::ParameterName, 'Date for the license copyright notice (defaults to the current year)')
            [CompletionResult]::new('-e', '-e', [CompletionResultType]::ParameterName, 'Regex pattern for files/directories to exclude. Applied during directory traversal')
            [CompletionResult]::new('--exclude', '--exclude', [CompletionResultType]::ParameterName, 'Regex pattern for files/directories to exclude. Applied during directory traversal')
            [CompletionResult]::new('--multiple', '--multiple', [CompletionResultType]::ParameterName, 'Enable support for multiple licenses in the same project (Default is replace)')
            [CompletionResult]::new('-A', '-A ', [CompletionResultType]::ParameterName, 'Do not respect the git_ignore file (If present in directory) and other pattern defaults')
            [CompletionResult]::new('--all', '--all', [CompletionResultType]::ParameterName, 'Do not respect the git_ignore file (If present in directory) and other pattern defaults')
            [CompletionResult]::new('-D', '-D ', [CompletionResultType]::ParameterName, 'Run without modification. See what would be changed')
            [CompletionResult]::new('--dry-run', '--dry-run', [CompletionResultType]::ParameterName, 'Run without modification. See what would be changed')
            [CompletionResult]::new('--prefer-block', '--prefer-block', [CompletionResultType]::ParameterName, 'When applying headers, which kind of comment token the user *wants* Completely possible line or block doesn''t exist, in which case it falls back to the other')
            [CompletionResult]::new('-v', '-v', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('--verbose', '--verbose', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('-q', '-q', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('--quiet', '--quiet', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('-h', '-h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', '--help', [CompletionResultType]::ParameterName, 'Print help')
            break
        }
        'lic;init' {
            [CompletionResult]::new('-v', '-v', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('--verbose', '--verbose', [CompletionResultType]::ParameterName, 'Increase logging verbosity')
            [CompletionResult]::new('-q', '-q', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('--quiet', '--quiet', [CompletionResultType]::ParameterName, 'Decrease logging verbosity')
            [CompletionResult]::new('-h', '-h', [CompletionResultType]::ParameterName, 'Print help')
            [CompletionResult]::new('--help', '--help', [CompletionResultType]::ParameterName, 'Print help')
            break
        }
        'lic;help' {
            [CompletionResult]::new('gen', 'gen', [CompletionResultType]::ParameterValue, 'Generate a license file')
            [CompletionResult]::new('unapply', 'unapply', [CompletionResultType]::ParameterValue, 'Remove license headers in source files')
            [CompletionResult]::new('apply', 'apply', [CompletionResultType]::ParameterValue, 'Apply license headers to source files')
            [CompletionResult]::new('init', 'init', [CompletionResultType]::ParameterValue, 'Initialize a default configuration file')
            [CompletionResult]::new('help', 'help', [CompletionResultType]::ParameterValue, 'Print this message or the help of the given subcommand(s)')
            break
        }
        'lic;help;gen' {
            break
        }
        'lic;help;unapply' {
            break
        }
        'lic;help;apply' {
            break
        }
        'lic;help;init' {
            break
        }
        'lic;help;help' {
            break
        }
    })

    $completions.Where{ $_.CompletionText -like "$wordToComplete*" } |
        Sort-Object -Property ListItemText
}
